import json
from dataclasses import dataclass


@dataclass
class VcapServices(object):

    services: dict

    @staticmethod
    def from_env_json(json_string: str):
        json_dict = dict(json.loads(json_string))
        vcap_services = VcapServices(services=json_dict)
        return vcap_services

    def does_service_exist(self, service_key: str) -> bool:
        return service_key in self.services.keys()

    def get_service_by_name(self, group_key: str, name: str) -> dict:
        service_group = self.services.get(group_key)
        if service_group:
            for service in service_group:
                if service.get("name") == name:
                    return service
            raise Exception(f"Service name '{name}' not found")
        raise Exception(f"Service group '{group_key}' not found")

    def get_service_credentials_value(self, group_key: str, name: str, key: str):
        service = self.get_service_by_name(group_key, name)
        return service.get("credentials").get(key)



import requests
from flask import current_app


def get_remote_data_as_json(endpoint):

    response = requests.get(endpoint)
    if response.status_code == 200:
        data = response.json()
        return data
    else:
        current_app.logger.warning(
            "GET remote data call was unsuccessful with status code:"
            f" {response.status_code} body: {response.text}."
        )
        response.raise_for_status()



from flask import current_app
from fsd_utils.mapping.application.application_utils import convert_bool_value
from fsd_utils.mapping.application.multi_input_utils import ProcessTypes


class MultiInput(ProcessTypes):
    @classmethod
    def indent(cls, indent_space):
        return " " * indent_space

    @classmethod
    def format_values(cls, value, index):
        return f"\n{cls.indent(5)}. {value}" if index != 1 else f". {value}"

    @classmethod
    def format_value_pair(cls, key, value, index):
        return (
            f"\n{cls.indent(5)}. {key}: {value}" if index != 1 else f". {key}: {value}"
        )

    @classmethod
    def format_keys_and_values(cls, key: str, value: list, index: enumerate):
        sanitised_values = convert_bool_value(value)
        values = "\n".join(
            [
                f"{cls.indent(5) if i == 1 else cls.indent(6)}. {str(item).strip()}"
                for i, item in enumerate(sanitised_values, start=1)
            ]
        )

        return (
            f"\n{cls.indent(5)}{str(key.strip())} \n {values}"  # noqa
            if index != 1
            else (f"{str(key.strip())} \n {values}")  # noqa
        )

    @classmethod
    def format_data(cls, sorted_data: dict) -> list:
        """
        Process a dictionary of sorted data, generating a list of formatted values.

        Args:
            sorted_data (dict): A dictionary of sorted data to process.

        Returns:
            list: A list of formatted values extracted from the input data.
        """

        output = []
        for index, (key, value) in enumerate(sorted_data.items(), start=1):

            if isinstance(key, int):
                key = str(key)
            try:
                uuid = ProcessTypes.is_valid_uuid(key)  # noqa
                if uuid:
                    output.append(cls.format_values(value, index))

                if isinstance(value, list):
                    output.append(cls.format_keys_and_values(key, value, index))
                if not uuid and not isinstance(value, list):
                    output.append(cls.format_value_pair(key, value, index))

            except Exception as e:
                current_app.logger.error(f"Error occurred while processing data: {e}")
                current_app.logger.error(
                    f"Couldn't format the muti input data for: {sorted_data}"
                )
        return output

    @classmethod
    def map_multi_input_data(cls, multi_input_data: list[dict]):
        """
        Maps and processes a list of input data, generating a formatted output.

        Args:
            multi_input_data (list[dict]): A list of dictionaries representing input data.

        Returns:
            str: A formatted output string containing processed data.

        Raises:
            Exception: If an error occurs during the mapping and processing.
        """

        try:
            sorted_data = {}
            for item in multi_input_data:
                if len(item) < 2:
                    if isinstance(item, str):
                        cls._str_single_item(item, sorted_data)
                    if isinstance(item, dict):
                        cls._dict_single_item(item, sorted_data)
                else:
                    if isinstance(item, dict):
                        cls._dict_items(item, sorted_data)

            output = cls.format_data(sorted_data)
            output = "\n".join(output)
            return output

        except Exception as e:
            current_app.logger.error(
                f"Error occurred while processing the multi input data: {e}"
            )
            current_app.logger.error(
                f"Couldn't map the multi input data for: {multi_input_data}"
            )



from typing import Tuple

from flask import current_app


class CheckerInterface:
    def check(self) -> Tuple[bool, str]:
        pass


class FlaskRunningChecker(CheckerInterface):
    def __init__(self):
        self.name = "check_flask_running"

    def check(self):
        if current_app:
            return True, "OK"
        else:
            return False, "Fail"


class DbChecker(CheckerInterface):
    def __init__(self, db):
        self.db = db
        self.name = "check_db"

    def check(self):
        from sqlalchemy.exc import SQLAlchemyError
        from sqlalchemy import text

        try:
            self.db.session.execute(text("SELECT 1"))
            return True, "OK"
        except SQLAlchemyError:
            current_app.logger.exception("DB Check failed")
            return False, "Fail"


class RedisChecker(CheckerInterface):
    def __init__(self, redis_client):  # TypeHint: FlaskRedis
        self.name = "check_redis"
        self.redis_client = redis_client

    def check(self):

        try:
            self.redis_client.client_list()
            return True, "OK"
        except Exception:
            current_app.logger.exception("Redis Check failed")
            return False, "Fail"
